/******************************************
Finding the maximum energy over all orientations
*******************************************/



# include "mex.h"        /* the algorithm is connect to matlab */
# include "math.h"


/* getting the index of matlab image, (x,y) location, (sx,sy) sizes */

int px(int x, int y, int sx, int sy)    
{            
   return (x + (y-1)*sx - 1); 
 }

# define BOUND 1e-5

double interp(double *I, double x, double y, int sx, int sy) 
{

    int x0, y0, i, j; 
    double a, d[2][2], w[2][2], sw; 

    x0 = floor(x); y0 = floor(y); 
    for (i=0; i<2; i++)
       for (j=0; j<2; j++) 
         {
          d[i][j] = sqrt((x-x0-i)*(x-x0-i)+(y-y0-j)*(y-y0-j)); 
          if (d[i][j] < BOUND)
             return (I[px(x0+i, y0+j, sx, sy)]); 
         }

    sw = 0.; 
    for (i=0; i<2; i++)
       for (j=0; j<2; j++) 
         {
          w[i][j] = 1./d[i][j]; 
          sw += w[i][j]; 
         }

    a = 0.; 
    for (i=0; i<2; i++)
       for (j=0; j<2; j++) 
         {
          a += I[px(x0+i, y0+j, sx, sy)]*w[i][j]/sw; 
         }
    return (a); 
}


double shift(double *I, int x, int y, int sx, int sy, double theta, double d)
{
     double xd, yd; 

     xd = d*cos(theta) + x; yd = d*sin(theta) + y; 
     return (interp(I, xd, yd, sx, sy)); 
}



# define PI 3.1415926

void Csketch(double **F, int N, 
          double *I, double *J, double *E, 
          int sx, int sy)
{
   int ind, h, x, y, a; 
   double m, s, r, theta;
    
   for (x=1; x<=sx; x++)
      for (y=1; y<=sy; y++)
       {
         h = px(x, y, sx, sy);
         m = 0.; s = 0.; a = 0;  
         for (ind=0; ind<N; ind++)
         {
           r = F[ind][h]; 
           s += r; 
           if (m<r)
                 {
                  m = r; a = ind; 
                 }
           I[h] = m;
           J[h] = (double)a*PI/(double)N;  
          }
      }

    for (x=2; x<sx; x++)
      for (y=2; y<sy; y++)
       {
         h = px(x, y, sx, sy);
         if ((I[h]>=shift(I, x, y, sx, sy, J[h], 1))&&
             (I[h]>=shift(I, x, y, sx, sy, J[h], -1)))
            E[h] = 1.; 
        }
}


/* the following is the mex file for communicating with matlab */

void mexFunction(int nlhs, mxArray *plhs[], 
                 int nrhs, const mxArray *prhs[])                
{
 int N, ind;           /* number of orientations */
 double **F;           /* filtered images at orientations */
 mxArray *f;           /* pointer to filtered images */


 double *I, *J, *E;        /* maximum energy, orientation, edge map */

 int sx, sy;               /* sizes of image */ 

 N = mxGetNumberOfElements(prhs[0]);  /* get total number of orientations */
 F = mxCalloc(N, sizeof(double*));    /* F: fitered images at all the orientations */
 
 for (ind=0; ind<N; ind++)
     {  
       f = mxGetCell(prhs[0], ind); 
       F[ind] = mxGetPr(f);              /* get filtered images at orientations */
     }


 I = mxGetPr(prhs[1]);    /* get the matlab pointer */

 sx = mxGetM(prhs[1]);    /* get size of x-direction */
 sy = mxGetN(prhs[1]);    /* get size of y-direction */

 J = mxGetPr(prhs[2]);    
 E = mxGetPr(prhs[3]);  

 Csketch(F, N, I, J, E, sx, sy); 
}

      



 

                    